<?php

require_once 'OferteoApi/OferteoApiUtils.php';
require_once 'OferteoApi/CategoriesHtmlBuilder.php';
require_once 'OferteoApi/LocationsHtmlBuilder.php';
require_once 'OferteoApi/RfpsHtmlBuilder.php';
require_once 'OferteoApi/RfpHtmlBuilder.php';
require_once 'OferteoApi/OferteoApiDict.php';

class OferteoApi {

    const HTTP_HEADER_OFERTEO_API_KEY = "X-Oferteo-ApiKey";

    const JSON_POSTFIX = ".json";
    const LOCATIONS_URL = "locations";
    const CATEGORIES_URL = "categories";
    const RFPS_URL = "rfps";
    const DEFAULT_SORT = 0;
    const DEFAULT_LOCALE = 'PL_pl';

    public $hostUrl;

    public $apiKey;

    public $urlPrefix;

    public $urlCurrent;

    public $urlQueryString;

    public $sort = 0;

    public $pageSize = 20;

    public $selectedCategory = null;

    public $selectedLocation = null;

    private $selectedCategoryUrl;

    private $selectedLocationUrl;

    private $selectedRfpId;

    public $page = null;

    public $categoriesTree;

    public $locationsTree;

    public $categoriesMap;

    public $locationsMap;

    public $locale = 'PL_pl';

    /**
     * Przyklady integracji: http://www.oferteo.pl/twoj-dzial-zlecen/przyklady-integracji
     *
     * @param $hostUrl - adres serwera Oferteo, np.: "http://api.oferteo.pl/v1/"
     * @param $apiKey - klucz autoryzacyjny otrzymany od Oferteo
     * @param $urlPrefix - kontener w ramach ktorego bedzie prezentowana lista zlecen, np. "/zlecenia"
     * @param $urlCurrent - aktualny URL, na podstawie ktorego beda odczytane wybrane lokalizacje, kategorie oraz zlecenia (opis linkow - patrz: {@link setUrlCurrent()})
     * @param $queryString - dodatkowe parametry przekazane w request
     */
    public function __construct($hostUrl, $apiKey, $urlPrefix, $urlCurrent, $queryString) {
        $this->hostUrl = $hostUrl;
        $this->apiKey = $apiKey;
        $this->urlPrefix = $urlPrefix;

        $categories = $this->getCategoriesTree();
        $this->fillCategoriesMap($categories);
        $locations = $this->getLocationsTree();
        $this->fillLocationsMap($locations);

        $this->updateCurrentUrl($urlCurrent);

        $this->setUrlQueryString($queryString);
    }

    /**
     * Aktualizacja URL, na podstawie ktorego odczytywane sa parametry zapytan.
     * Dla przegladania listy zlecen URL moze sie skladac z nastepujacych elementow:<br>
     * <code>prefix[/url_kategorii][/url_lokalizacji[/pager]</code>
     * <ul>
     * 	<li><strong>prefix</strong> - prefix URL-a, bedacy kontenerem, w ramach ktorego beda generowane linki, np. /zlecenia </li>
     * 	<li><strong>url_kategorii</strong> - identyfikator kategorii w formacie "user-friendly URL" </li>
     * 	<li><strong>url_lokalizacji</strong> - identyfikator wybranej lokalizacji w formacie "user-friendly URL"</li>
     * 	<li><strong>pager</strong> - numer strony z wynikami (liczba 1...)
     * </ul>
     * Przyklady:<br>
     * <ul>
     * 	<li>/zlecenia/dolnoslaskie</li>
     * 	<li>/zlecenia/budownictwo-uslugi</li>
     * 	<li>/zlecenia/budownictwo-uslugi/dolnoslaskie</li>
     * 	<li>/zlecenia/budownictwo-uslugi/dolnoslaskie/2</li>
     * </ul>
     *
     * Dla podgladu pojedynczego zlecenia URL moze sie skladac z nastepujacych elementow:<br>
     * <code>prefix/rfp_id</code>
     * <ul>
     * 	<li><strong>prefix</strong> - prefix URL-a, bedacy kontenerem, w ramach ktorego beda generowane linki, np. /zlecenia </li>
     * 	<li><strong>rfp_id</strong> - identyfikator zlecenia w formacie "user-friendly URL" </li>
     * </ul>
     * Przyklad:<br>
     * <ul>
     * 	<li>/zlecenia/zlece-wykonanie-elewacji-na-domu-pow-ok-200m2-012345</li>
     * </ul>
     * <br>
     * URL-e w takiej postaci sa generowane przez OferteoApi w metodach budujacych przykladowy html.
     * Przyklady integracji: http://www.oferteo.pl/twoj-dzial-zlecen/przyklady-integracji
     * @param $url
     */
    public function setUrlCurrent($url) {
        $this->updateCurrentUrl($url);
    }

    /**
     * Ustawienie parametrow dodatkowych, ktore beda uzyte przy wywolywaniu zapytań, np:
     * <ul>
     * 	<li> sort - rodzaj sortowania
     * </ul>
     * @return
     */
    public function setUrlQueryString($urlQueryString) {
        $this->urlQueryString = $urlQueryString;
        $this->sort = OferteoApiUtils::getSortFromUrl($urlQueryString);
    }

    /**
     * Wykonuje zapytanie w celu pobrania listy zapytan (RFP).
     * Parametry zapytania sa ustawione na podstawie aktualnego URL-a (patrz: {@link #setUrlCurrent(String)})
     * @return
     */
    public function getRfps() {
        return $this->getJsonDataObject($this->buildIndexUrl(self::RFPS_URL));
    }

    /**
     * Wywoluje serwis Oferteo w celu pobrania dostepnych kategorii.
     * Parametry pomocnicze ustawione na podstawie urlCurrent (szczegoly: {@link #setUrlCurrent()})
     * @return
     */
    public function getCategoriesTree() {
        return $this->getJsonDataObject($this->buildIndexUrl(self::CATEGORIES_URL));
    }

    public function fillCategoriesMap($categories) {
        foreach ($categories as $category) {
            $this->categoriesMap[$category->urlCategory] = $category->id;
            $this->fillCategoriesMap($category->subcategories);
        }
    }

    /**
     * Wywoluje serwis Oferteo w celu pobrania dostepnych lokalizacji.
     * Parametry pomocnicze ustawione na podstawie urlCurrent (szczegoly: {@link #setUrlCurrent()})
     * @return
     */
    public function getLocationsTree() {
        return $this->getJsonDataObject($this->buildIndexUrl(self::LOCATIONS_URL));
    }

    public function fillLocationsMap($locations) {
        foreach ($locations as $location) {
            $this->locationsMap[$location->urlLocation] = $location->id;
            $this->fillLocationsMap($location->locations);
        }
    }

    private function buildIndexUrl($restUrl) {
        return $this->hostUrl . $restUrl . self::JSON_POSTFIX
            . "?categoryId=" . ($this->selectedCategory != null ? $this->selectedCategory : "")
            . "&locationId=" . ($this->selectedLocation != null ? $this->selectedLocation : "")
            . "&pageNumber=" . ($this->page > 0 ? $this->page : 1)
            . "&pageSize=" . $this->pageSize
            . "&sort=" . $this->sort;
    }

    private function buildShowUrl($url, $selectedId) {
        return $this->hostUrl . $url . "/" . $selectedId . self::JSON_POSTFIX;
    }

    public function getJsonDataObject($jsonUrl) {
        try {
            $data = $this->getJsonData($jsonUrl);
            if ($data == null) {
                return null;
            }
        } catch (Exception $e) {
            echo($e);
            return null;
        }

        return $data;
    }

    public function getJsonData($jsonUrl) {
        $headers = array(
            self::HTTP_HEADER_OFERTEO_API_KEY . ': ' . $this->apiKey
        );

        if (function_exists('curl_init')) {
            $ch = curl_init();

            curl_setopt($ch, CURLOPT_URL, $jsonUrl);
            curl_setopt($ch, CURLOPT_POST,   0);
            curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1 );
            curl_setopt($ch, CURLOPT_HTTPHEADER, $headers);

            $results = curl_exec($ch);

            curl_close($ch);
        } else {
            $jsonUrl .= (strpos($jsonUrl, '?') === false ? '?' : '&') . 'apiKey=' . $this->apiKey;

            $results = @file($jsonUrl);
        }

        if (!$results) {
            throw new Exception('Nie udało się nawiązać połączenia.');
        }

        return json_decode(is_array($results) ? $results[0] : $results);
    }

    private function updateCurrentUrl($newUrl) {
        $this->updateSelected(
            $newUrl,
            OferteoApiUtils::findCategoryInUrl($newUrl, $this->urlPrefix, $this->categoriesMap),
            OferteoApiUtils::findLocationInUrl($newUrl, $this->urlPrefix, $this->locationsMap),
            OferteoApiUtils::findRfpIdInUrl($newUrl),
            OferteoApiUtils::findPagerInUrl($newUrl)
        );
    }

    private function updateSelected($urlCurrent, $categoryUrl, $locationUrl, $rfpId, $page) {
        $this->urlCurrent = $urlCurrent;
        $this->selectedCategoryUrl = $categoryUrl;
        $this->selectedLocationUrl = $locationUrl;
        $this->selectedRfpId  = $rfpId;
        $this->page = $page;

        //ustawienie wybranej kategorii i lokalizacji na podstawie danych z url-a
        if ($this->selectedCategoryUrl != null) {
            $this->selectedCategory = $this->categoriesMap[$this->selectedCategoryUrl];
        } else {
            $this->selectedCategory = null;
        }
        if ($this->selectedLocationUrl != null) {
            $this->selectedLocation = $this->locationsMap[$this->selectedLocationUrl];
        } else {
            $this->selectedLocationUrl = null;
        }
    }

    /**
     * Na podstawie podanych kategorii buduje HTML przedstawiajacy kategorie w postaci listy zbudowanej z elementow {@literal <ul> oraz <li>}.
     * @param $categories - kolekcja kategorii najwyzszego poziomu
     * @return
     */
    public function getCategoryHtml($categories) {
        $htmlBuilder = new CategoriesHtmlBuilder($categories, $this->urlPrefix, $this->selectedLocationUrl);
        return $htmlBuilder->getHtml();
    }

    /**
     * Na podstawie podanych lokalizacji buduje HTML przedstawiajacy lokalizacje w postaci listy zbudowanej z elementow {@literal <ul> oraz <li>}.
     * @param $locations - kolekcja lokalizacji najwyzszego poziomu
     * @return
     */
    public function getLocationHtml($locations) {
        $htmlBuilder = new LocationsHtmlBuilder($locations, $this->urlPrefix, $this->selectedCategoryUrl);
        return $htmlBuilder->getHtml();
    }

    public function showingSelectedRfp() {
        return $this->selectedRfpId > 0;
    }

    /**
     * Pobiera szczegoly zapytania o podanym id
     * @param idRfp
     * @return null jesli <code>idRfp &lt; 0 </code> lub jesli nie udalo sie wczytac danych RFP
     */
    public function getRfp($idRfp = null) {
        if (!$idRfp) {
            $idRfp = $this->selectedRfpId;
        }

        return $this->getJsonDataObject($this->buildShowUrl(self::RFPS_URL, $idRfp));
    }

    /**
     * Na podstawie obiektu z wynikami wyszukiwania zapytan ofertowych buduje HTML przedstawiacy dane w postaci listy
     * podzielonej na zestawy po 20 elementow.
     * @param $rfpSearchResult - obiekt z wynikami wyszukiwania ({@link #getRfps()}).
     * @param $pageSize - liczba elementow na jednej stronie
     * @return
     */
    public function getRfpsListHtml($rfpSearchResult, $pageSize = 20) {
        $htmlBuilder = new RfpsHtmlBuilder(
            $rfpSearchResult,
            $this->urlPrefix,
            $this->locale,
            $this->urlCurrent,
            $this->selectedCategoryUrl,
            $this->selectedLocationUrl,
            $pageSize,
            $this->sort,
            $this->urlQueryString
        );
        return $htmlBuilder->getHtml();
    }

    /**
     * Ustawia parametry dla zapytan na podstawie wybranego RFP:
     * <ul>
     * 	<li>categoryUrl - na podstawie kategorii wybranego RFP</li>
     * 	<li>voivodShipUrl - na podstawie wojewodztwa wybranego RFP</li>
     * </ul>
     * Dzieki temu nastepne zapytania po kategorie i lokalizacje beda mialy
     * ustawione parametry ustawione na podstawie aktualnego zlecenia
     * @param rfp - wybrany RFP do prezentowania szczegolow
     */
    public function updateSelectedCategoryAndLocation($rfp = null) {
        if (!$rfp) {
            return;
        }

        $this->updateSelected(
            $this->urlCurrent,
            $rfp->categoryUrl,
            $rfp->dstVoivodshipUrl,
            $this->selectedRfpId,
            $this->page
        );
    }

    /**
     * Na podstawie obiektu reprezentujacego pojedyncze zapytanie buduje HTML prezentujacy szczegoly zapytania
     * @param $apiRfp
     * @return
     */
    public function getRfpHtml($apiRfp) {
        $htmlBuilder = new RfpHtmlBuilder($apiRfp, $this->urlPrefix, $this->locale, $this->urlCurrent);
        return $htmlBuilder->getHtml();
    }
}